from leakix.client import Client, Scope, HostResult
from leakix.field import *
from leakix.plugin import *
from leakix.query import *
from leakix.response import *
from leakix.client import __VERSION__



from abc import ABCMeta, abstractmethod
from typing import Optional, List
from leakix.field import CustomField


class AbstractQuery(metaclass=ABCMeta):
    """
    An abstract query. Should not be instantiated.
    """

    @abstractmethod
    def serialize(self) -> str:
        pass


class EmptyQuery(AbstractQuery):
    """
    Represents an empty query, which will be translated to a wildcard. Using an empty query means applying no filter on
    the results and the outputs of the query will be the latest results the engine found.
    """

    def serialize(self) -> str:
        return "*"


class Query(AbstractQuery):
    """
    A query for a custom field. Fields can be found on https://docs.leakix.net/docs/query/fields/
    A list of fields can be found in `field.py`.
    """

    def __init__(self, field: CustomField):
        self.field = field


class MustQuery(Query):
    """
    A MustQuery is a query that will impose a condition on the field, and will be translated to a `+query` in the API
    A list of fields can be found in `field.py`.
    """

    def serialize(self) -> str:
        return "+%s" % self.field.serialize()


class MustNotQuery(Query):
    """
    A MustNotQuery is a query that will exclude a condition on the field, and will be translated to a `-query` in the API
    A list of fields can be found in `field.py`.
    """

    def serialize(self) -> str:
        return "-%s" % self.field.serialize()


class ShouldQuery(Query):
    """
    A ShouldQuery is a query that will make the condition optional on the field, and will be translated to `query` in
    the API.
    A list of fields can be found in `field.py`.
    """

    def serialize(self) -> str:
        return "%s" % self.field.serialize()


class RawQuery(AbstractQuery):
    """
    A RawQuery is a query that can be used as on the website. For instance, to filter on the hosts `.be`, you can use
    RawQuery("+host:.be").
    """

    def __init__(self, raw_q: str):
        self.raw_q = raw_q

    def serialize(self) -> str:
        return self.raw_q



import json
from typing import Optional, List
import requests
from l9format import l9format
from enum import Enum
from serde import Model, fields
from leakix.response import SuccessResponse, ErrorResponse, RateLimitResponse
from leakix.query import *
from leakix.plugin import *
from leakix.plugin import APIResult
from leakix.field import *


__VERSION__ = "0.1.9"


class Scope(Enum):
    SERVICE = "service"
    LEAK = "leak"


class HostResult(Model):
    Services: fields.Optional(fields.List(fields.Nested(l9format.L9Event)))
    Leaks: fields.Optional(fields.List(fields.Nested(l9format.L9Event)))


DEFAULT_URL = "https://leakix.net"


class Client:
    MAX_RESULTS_PER_PAGE = 20

    def __init__(
        self,
        api_key: Optional[str] = None,
        base_url: Optional[str] = DEFAULT_URL,
    ):
        self.api_key = api_key
        self.base_url = base_url if base_url else DEFAULT_URL
        self.headers = {
            "Accept": "application/json",
            "User-agent": "leakix-client-python/%s" % __VERSION__,
        }
        if api_key:
            self.headers["api-key"] = api_key

    def __get(self, url, params):
        r = requests.get(
            url,
            params=params,
            headers=self.headers,
        )
        if r.status_code == 200:
            response_json = r.json() if r.content else []
            return SuccessResponse(response=r, response_json=response_json)
        elif r.status_code == 429:
            return RateLimitResponse(response=r)
        elif r.status_code == 204:
            return ErrorResponse(response=r, response_json=[], status_code=200)
        else:
            return ErrorResponse(response=r, response_json=r.json())

    def get(self, scope: Scope, queries: Optional[List[Query]] = None, page: int = 0):
        """
        The function takes a scope (either "leaks" or "services"). The value can be constructed using `Scope.SERVICE` or
        `Scope.LEAK`.
        The second parameter is a list of queries. The default value will be considered as the wildcard `*`, which is
        also the default value on the web interface.
        Structured queries can be built using the different classes in `query.py` and `field.py`.
        If you want to build "raw" queries, like on the search bar on the website, use the class `RawQuery`.

        The output will be an abstract response, which can be a successfull HTTP response (represented by the class
        `SuccessResponse`) or a failed HTTP response (represented by the class `ErrorResponse`).
        Methods `is_success` and `is_error` are provided to the user to verify in their application what the state of
        the response is.

        The output of a query can be accessed using the method `json`, for instance `response.json()`.
        In the case of a successfull response, the output will be a list of L9Event.
        When you have an object of type `l9Event` (or the longer
        `l9format.l9format.L9Event`), you can refer to
        [L9Event](https://github.com/LeakIX/l9format-python/blob/main/l9format/l9format.py#L158)
        model class for the available fields.
        For instance, to access the IP of an object `event` of type `L9Event`, you can
        use `event.ip`.
        """
        if page < 0:
            raise ValueError("Page argument must be a positive integer")
        if queries is None or len(queries) == 0:
            serialized_query = EmptyQuery().serialize()
        else:
            serialized_query = [q.serialize() for q in queries]
            serialized_query = " ".join(serialized_query)
            serialized_query = "%s" % serialized_query
        url = "%s/search" % self.base_url
        r = self.__get(
            url=url, params={"scope": scope.value, "q": serialized_query, "page": page}
        )
        return r

    def get_service(self, queries: Optional[List[Query]] = None, page: int = 0):
        """
        Shortcut for `get` with the scope `Scope.Service`.

        """
        r = self.get(Scope.SERVICE, queries=queries, page=page)
        if r.is_success():
            r.response_json = [
                l9format.L9Event.from_dict(res) for res in r.response_json
            ]
        return r

    def get_leak(self, queries: Optional[List[Query]] = None, page: int = 0):
        """
        Shortcut for `get` with the scope `Scope.Leak`.
        """
        r = self.get(Scope.LEAK, queries=queries, page=page)
        if r.is_success():
            r.response_json = [
                l9format.L9Event.from_dict(res) for res in r.response_json
            ]
        return r

    def get_host(self, ipv4: str):
        """
        Returns the list of services and associated leaks for a given host. Only the ipv4 format is supported at the
        moment.
        """
        url = "%s/host/%s" % (self.base_url, ipv4)
        r = self.__get(url, params=None)
        if r.is_success():
            response_json = r.json()
            formatted_result = HostResult.from_dict(response_json)
            response_json = {
                "services": formatted_result.Services,
                "leaks": formatted_result.Leaks,
            }
            r.response_json = response_json
        return r

    def get_plugins(self):
        """
        Returns the list of plugins the authenticated user with the given API key has access to.

        The output is a list of `APIResult` objects. The fields are `name` which is the plugin name, and `description`
        which contains a brief description of the plugin.
        Paid users have access to a broader list of plugins. The full list you can be found on
        https://leakix.net/plugins.
        For the paid plans, have a look at https://leakix.net/plans.
        """
        url = "%s/api/plugins" % (self.base_url)
        r = self.__get(url, params=None)
        if r.is_success():
            r.response_json = [APIResult.from_dict(d) for d in r.json()]
        return r

    def bulk_export(self, queries: Optional[List[Query]] = None):
        url = "%s/bulk/search" % (self.base_url)
        if queries is None or len(queries) == 0:
            serialized_query = EmptyQuery().serialize()
        else:
            serialized_query = [q.serialize() for q in queries]
            serialized_query = " ".join(serialized_query)
            serialized_query = "%s" % serialized_query
        params = {"q": serialized_query}
        r = requests.get(url, params=params, headers=self.headers, stream=True)
        if r.status_code == 200:
            response_json = []
            for line in r.iter_lines():
                json_event = json.loads(line)
                response_json.append(l9format.L9Aggregation.from_dict(json_event))
            return SuccessResponse(response=r, response_json=response_json)
        elif r.status_code == 429:
            return RateLimitResponse(response=r)
        elif r.status_code == 204:
            return ErrorResponse(response=r, response_json=[], status_code=200)
        else:
            return ErrorResponse(response=r, response_json=r.json())
        return r

    def bulk_service(self, queries: Optional[List[Query]] = None):
        url = "%s/bulk/service" % (self.base_url)
        if queries is None or len(queries) == 0:
            serialized_query = EmptyQuery().serialize()
        else:
            serialized_query = [q.serialize() for q in queries]
            serialized_query = " ".join(serialized_query)
            serialized_query = "%s" % serialized_query
        params = {"q": serialized_query}
        r = requests.get(url, params=params, headers=self.headers, stream=True)
        if r.status_code == 200:
            response_json = []
            for line in r.iter_lines():
                json_event = json.loads(line)
                response_json.append(l9format.L9Event.from_dict(json_event))
            return SuccessResponse(response=r, response_json=response_json)
        elif r.status_code == 429:
            return RateLimitResponse(response=r)
        elif r.status_code == 204:
            return ErrorResponse(response=r, response_json=[], status_code=200)
        else:
            return ErrorResponse(response=r, response_json=r.json())
        return r



from abc import ABCMeta, abstractmethod


class AbstractResponse(metaclass=ABCMeta):
    def __init__(self, response, response_json=None, status_code=None):
        self.response = response
        self._status_code = (
            status_code if status_code is not None else self.response.status_code
        )
        self.response_json = (
            response_json if response_json is not None else response.json()
        )

    def json(self):
        return self.response_json

    def status_code(self):
        return self._status_code

    @abstractmethod
    def is_success(self):
        pass

    @abstractmethod
    def is_error(self):
        pass


class SuccessResponse(AbstractResponse):
    def is_success(self):
        return True

    def is_error(self):
        return False


class ErrorResponse(AbstractResponse):
    def is_success(self):
        return False

    def is_error(self):
        return True


class RateLimitResponse(ErrorResponse):
    pass


class R(AbstractResponse):
    def __init__(self, response, response_json=None, status_code=None):
        super(R, self).__init__(
            response, response_json=response_json, status_code=status_code
        )



from enum import Enum
from serde import Model, fields


class APIResult(Model):
    name: fields.Str()
    description: fields.Str()


class Plugin(Enum):
    """
    The list of plugins can be found on https://leakix.net/plugins. Some plugins are only available to paid users. Have
    a look at https://leakix.net/pricing for a detailled list.
    """

    ApacheStatusHttpPlugin = "ApacheStatusHttpPlugin"
    BitbucketPlugin = "BitbucketPlugin"
    CheckMkPlugin = "CheckMkPlugin"
    ConfigJsonHttp = "ConfigJsonHttp"
    ConfluenceVersionIssue = "ConfluenceVersionIssue"
    Consul = "Consul"
    CouchDbOpenPlugin = "CouchDbOpenPlugin"
    DeadMon = "DeadMon"
    DockerRegistryHttpPlugin = "DockerRegistryHttpPlugin"
    DotDsStoreOpenPlugin = "DotDsStoreOpenPlugin"
    DotEnvConfigPlugin = "DotEnvConfigPlugin"
    ElasticSearchOpenPlugin = "ElasticSearchOpenPlugin"
    ExchangeVersion = "ExchangeVersion"
    FortiOSPlugin = "FortiOSPlugin"
    GitConfigHttpPlugin = "GitConfigHttpPlugin"
    GrafanaOpenPlugin = "GrafanaOpenPlugin"
    HiSiliconDVR = "HiSiliconDVR"
    HttpNTLM = "HttpNTLM"
    JenkinsOpenPlugin = "JenkinsOpenPlugin"
    JiraPlugin = "JiraPlugin"
    KafkaOpenPlugin = "KafkaOpenPlugin"
    LaravelTelescopeHttpPlugin = "LaravelTelescopeHttpPlugin"
    Log4JOpportunistic = "Log4JOpportunistic"
    MetabaseHttpPlugin = "MetabaseHttpPlugin"
    MongoOpenPlugin = "MongoOpenPlugin"
    MysqlOpenPlugin = "MysqlOpenPlugin"
    PaloAltoPlugin = "PaloAltoPlugin"
    PhpInfoHttpPlugin = "PhpInfoHttpPlugin"
    PhpStdinPlugin = "PhpStdinPlugin"
    ProxyOpenPlugin = "ProxyOpenPlugin"
    QnapVersion = "QnapVersion"
    RedisOpenPlugin = "RedisOpenPlugin"
    SmbPlugin = "SmbPlugin"
    SonarQubePlugin = "SonarQubePlugin"
    SonicWallSMAPlugin = "SonicWallSMAPlugin"
    SophosPlugin = "SophosPlugin"
    SymfonyProfilerPlugin = "SymfonyProfilerPlugin"
    SymfonyVerbosePlugin = "SymfonyVerbosePlugin"
    TraversalHttpPlugin = "TraversalHttpPlugin"
    veeaml9 = "veeaml9"
    WpUserEnumHttp = "WpUserEnumHttp"
    YiiDebugPlugin = "YiiDebugPlugin"
    ZimbraPlugin = "ZimbraPlugin"
    ZookeeperOpenPlugin = "ZookeeperOpenPlugin"
    ZyxelVersion = "ZyxelVersion"



from leakix.plugin import Plugin
from abc import ABCMeta, abstractmethod
from datetime import datetime
from typing import Optional
from enum import Enum


class Operator(Enum):
    StrictlyGreater = ">"
    StrictlySmaller = "<"
    Equal = ""


class CustomField:
    def __init__(self, v: str, field_name: str, operator: Optional[Operator] = None):
        if operator is None:
            operator = Operator.Equal
        self.operator = operator
        self.field_name = field_name
        self.v = v

    def serialize(self) -> str:
        if self.operator != Operator.Equal:
            res = "%s:%s%s" % (self.field_name, self.operator.value, self.v)
        else:
            res = "%s:%s" % (self.field_name, self.v)
        return res


class TimeField(CustomField):
    def __init__(self, d: datetime, operator: Optional[Operator] = None):
        v = '"%s"' % d.strftime("%Y-%m-%d")
        super(TimeField, self).__init__(v=v, operator=operator, field_name="time")


class UpdateDateField(CustomField):
    def __init__(self, d: datetime, operator: Optional[Operator] = None):
        # v = '"%s"' % d.strftime("%Y-%m-%d %H:%M:%S")
        v = '"%s"' % d.strftime("%Y-%m-%d")
        super(UpdateDateField, self).__init__(
            v=v, operator=operator, field_name="update_date"
        )


class AgeField(CustomField):
    def __init__(self, age: int, operator: Optional[Operator] = None):
        super(AgeField, self).__init__(v=str(age), operator=operator, field_name="age")


class PluginField(CustomField):
    def __init__(self, p: Plugin):
        v = p.value
        super(PluginField, self).__init__(v=v, operator=None, field_name="plugin")


class IPField(CustomField):
    def __init__(self, ip: str, operator: Optional[Operator] = None):
        super(IPField, self).__init__(v=ip, operator=operator, field_name="ip")


class PortField(CustomField):
    def __init__(self, port: int, operator: Optional[Operator] = None):
        assert 0 <= port < 65536
        super(PortField, self).__init__(
            v=str(port), operator=operator, field_name="port"
        )


class CountryField(CustomField):
    def __init__(self, country: str):
        super(CountryField, self).__init__(
            v=country, operator=None, field_name="country"
        )
